#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 The Aerospace Corporation.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import analog
from gnuradio import blocks
from gnuradio import channels
from gnuradio import digital
from gnuradio import eng_notation
from gnuradio import fec
from gnuradio import filter
from gnuradio import gr
from gnuradio import gr, blocks, filter
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from optparse import OptionParser
import goessdr
import numpy as np
import osmosdr
import pmt
import time


class goes_lrit_emwin_nogui(gr.top_block):

    def __init__(self, callback_args=[], decim=1, device_args='uhd,addr=192.168.10.3,otw_format=sc16', digital_fc=0, emwindir='.', carrier_fc=137.5e3, lritdir='.', nlevel=0, report_callback='lambda x:x', report_interval=10, samp_rate=1e6, symb_rate=500e3):
        gr.top_block.__init__(self, "Goes Lrit Emwin Nogui")

        ##################################################
        # Parameters
        ##################################################
        self.callback_args = callback_args
        self.decim = decim
        self.device_args = device_args
        self.digital_fc = digital_fc
        self.emwindir = emwindir
        self.carrier_fc = carrier_fc
        self.lritdir = lritdir
        self.nlevel = nlevel
        self.report_callback = report_callback
        self.report_interval = report_interval
        self.samp_rate = samp_rate
        self.symb_rate = symb_rate

        ##################################################
        # Variables
        ##################################################
        self.sps = sps = samp_rate / symb_rate / decim
        self.rrc_order = rrc_order = 64
        self.pfb_nfilt = pfb_nfilt = 32
        
        self.channelizer_filter_taps = channelizer_filter_taps = firdes.low_pass(1.0, 1, 0.5  / decim, 0.25 / decim, firdes.WIN_HAMMING, 6.76)
          
        
        
        self.variable_cc_decoder_def_0 = variable_cc_decoder_def_0 = fec.cc_decoder.make(2 ** 15, 7, 2, ([79, 109]), -1, -1, fec.CC_STREAMING, False)
            
        self.pll_loopbw = pll_loopbw = 0.02
        
        self.pfb_taps = pfb_taps = firdes.root_raised_cosine(pfb_nfilt, pfb_nfilt, 1.0 / sps, 0.5, rrc_order * pfb_nfilt)
          
        self.channelizer_taps = channelizer_taps = (1,) if decim == 1 else channelizer_filter_taps

        ##################################################
        # Blocks
        ##################################################
        self.single_pole_iir_filter_xx_0 = filter.single_pole_iir_filter_ff(0.00001, 1)
        self.root_raised_cosine_filter_0 = filter.fir_filter_ccf(1, firdes.root_raised_cosine(
        	1, samp_rate /  decim, symb_rate, .5, rrc_order))
        self.probe_foffset = blocks.probe_signal_f()
        self.osmosdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + device_args )
        self.osmosdr_source_0.set_sample_rate(samp_rate)
        self.osmosdr_source_0.set_center_freq(carrier_fc, 0)
        self.osmosdr_source_0.set_freq_corr(0, 0)
        self.osmosdr_source_0.set_dc_offset_mode(0, 0)
        self.osmosdr_source_0.set_iq_balance_mode(0, 0)
        self.osmosdr_source_0.set_gain_mode(False, 0)
        self.osmosdr_source_0.set_gain(10, 0)
        self.osmosdr_source_0.set_if_gain(20, 0)
        self.osmosdr_source_0.set_bb_gain(20, 0)
        self.osmosdr_source_0.set_antenna('', 0)
        self.osmosdr_source_0.set_bandwidth(0, 0)
          
        self.mpc_mult = goessdr.message_passing_control_wrapper(blocks.multiply_const_cc, 1)
          
        self.mpc_delay = goessdr.message_passing_control_wrapper(blocks.delay, gr.sizeof_gr_complex, 0)
          
        self.goessdr_goes_sink_0 = goessdr.goes_sink(1000, 4000, lritdir, emwindir)
        self.goessdr_callback_message_sink_0 = goessdr.callback_message_sink(report_callback, *callback_args, **{})
          
        self.fir_channelizer = filter.freq_xlating_fir_filter_ccc(decim, (channelizer_taps), digital_fc, samp_rate)
        self.fec_extended_decoder_0 = fec.extended_decoder(decoder_obj_list=variable_cc_decoder_def_0, threading= None, ann=None, puncpat='11', integration_period=10000)
        self.digital_probe_mpsk_snr_est_c_0 = digital.probe_mpsk_snr_est_c(2, int(report_interval * symb_rate), 0.001)
        self.digital_costas_loop_cc_0 = digital.costas_loop_cc(np.pi / 100., 2, False)
        self.digital_clock_recovery_mm_xx_0 = digital.clock_recovery_mm_cc(sps, (np.pi / 100) ** 2, 0, np.pi / 100, 0.01)
        self.channels_channel_model_0 = channels.channel_model(
        	noise_voltage=nlevel,
        	frequency_offset=0.0,
        	epsilon=1.0,
        	taps=(1.0, ),
        	noise_seed=0,
        	block_tags=False
        )
        self.blocks_unpacked_to_packed_xx_0 = blocks.unpacked_to_packed_bb(1, gr.GR_MSB_FIRST)
        self.blocks_probe_rate_0 = blocks.probe_rate(gr.sizeof_gr_complex*1, report_interval * 1000, 0.0001)
        self.blocks_multiply_xx_0 = blocks.multiply_vcc(1)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_vff((samp_rate / decim / np.pi / 2 / 2, ))
        self.blocks_message_strobe_0 = blocks.message_strobe(pmt.cons(pmt.intern("report"), pmt.PMT_T), report_interval * 1000)
        self.blocks_complex_to_real_0_0 = blocks.complex_to_real(1)
        self.analog_pll_freqdet_cf_0 = analog.pll_freqdet_cf(2 * np.pi * pll_loopbw, 20 * np.pi * pll_loopbw, -20 * np.pi * pll_loopbw)
        self.analog_agc_xx_0 = analog.agc_cc(1e-4, 1, 1.0)
        self.analog_agc_xx_0.set_max_gain(0)

        ##################################################
        # Connections
        ##################################################
        self.msg_connect((self.blocks_message_strobe_0, 'strobe'), (self.goessdr_goes_sink_0, 'report'))    
        self.msg_connect((self.blocks_probe_rate_0, 'rate'), (self.goessdr_callback_message_sink_0, 'msg'))    
        self.msg_connect((self.digital_probe_mpsk_snr_est_c_0, 'snr'), (self.goessdr_callback_message_sink_0, 'msg'))    
        self.msg_connect((self.goessdr_goes_sink_0, 'report'), (self.goessdr_callback_message_sink_0, 'msg'))    
        self.msg_connect((self.goessdr_goes_sink_0, 'control'), (self.mpc_delay, 'set'))    
        self.msg_connect((self.goessdr_goes_sink_0, 'control'), (self.mpc_mult, 'set'))    
        self.connect((self.analog_agc_xx_0, 0), (self.blocks_multiply_xx_0, 0))    
        self.connect((self.analog_agc_xx_0, 0), (self.blocks_multiply_xx_0, 1))    
        self.connect((self.analog_agc_xx_0, 0), (self.digital_costas_loop_cc_0, 0))    
        self.connect((self.analog_pll_freqdet_cf_0, 0), (self.single_pole_iir_filter_xx_0, 0))    
        self.connect((self.blocks_complex_to_real_0_0, 0), (self.fec_extended_decoder_0, 0))    
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.probe_foffset, 0))    
        self.connect((self.blocks_multiply_xx_0, 0), (self.analog_pll_freqdet_cf_0, 0))    
        self.connect((self.blocks_unpacked_to_packed_xx_0, 0), (self.goessdr_goes_sink_0, 0))    
        self.connect((self.channels_channel_model_0, 0), (self.analog_agc_xx_0, 0))    
        self.connect((self.digital_clock_recovery_mm_xx_0, 0), (self.blocks_probe_rate_0, 0))    
        self.connect((self.digital_clock_recovery_mm_xx_0, 0), (self.digital_probe_mpsk_snr_est_c_0, 0))    
        self.connect((self.digital_clock_recovery_mm_xx_0, 0), (self.mpc_delay, 0))    
        self.connect((self.digital_costas_loop_cc_0, 0), (self.root_raised_cosine_filter_0, 0))    
        self.connect((self.fec_extended_decoder_0, 0), (self.blocks_unpacked_to_packed_xx_0, 0))    
        self.connect((self.fir_channelizer, 0), (self.channels_channel_model_0, 0))    
        self.connect((self.mpc_delay, 0), (self.mpc_mult, 0))    
        self.connect((self.mpc_mult, 0), (self.blocks_complex_to_real_0_0, 0))    
        self.connect((self.osmosdr_source_0, 0), (self.fir_channelizer, 0))    
        self.connect((self.root_raised_cosine_filter_0, 0), (self.digital_clock_recovery_mm_xx_0, 0))    
        self.connect((self.single_pole_iir_filter_xx_0, 0), (self.blocks_multiply_const_vxx_0, 0))    

    def get_callback_args(self):
        return self.callback_args

    def set_callback_args(self, callback_args):
        self.callback_args = callback_args

    def get_decim(self):
        return self.decim

    def set_decim(self, decim):
        self.decim = decim
        self.set_sps(self.samp_rate / self.symb_rate / self.decim)
        self.set_channelizer_taps((1,) if self.decim == 1 else self.channelizer_filter_taps)
        self.root_raised_cosine_filter_0.set_taps(firdes.root_raised_cosine(1, self.samp_rate /  self.decim, self.symb_rate, .5, self.rrc_order))
        self.blocks_multiply_const_vxx_0.set_k((self.samp_rate / self.decim / np.pi / 2 / 2, ))

    def get_device_args(self):
        return self.device_args

    def set_device_args(self, device_args):
        self.device_args = device_args

    def get_digital_fc(self):
        return self.digital_fc

    def set_digital_fc(self, digital_fc):
        self.digital_fc = digital_fc
        self.fir_channelizer.set_center_freq(self.digital_fc)

    def get_emwindir(self):
        return self.emwindir

    def set_emwindir(self, emwindir):
        self.emwindir = emwindir
        self.goessdr_goes_sink_0.set_output_dirs(self.lritdir, self.emwindir)

    def get_carrier_fc(self):
        return self.carrier_fc

    def set_carrier_fc(self, carrier_fc):
        self.carrier_fc = carrier_fc
        self.osmosdr_source_0.set_center_freq(self.carrier_fc, 0)

    def get_lritdir(self):
        return self.lritdir

    def set_lritdir(self, lritdir):
        self.lritdir = lritdir
        self.goessdr_goes_sink_0.set_output_dirs(self.lritdir, self.emwindir)

    def get_nlevel(self):
        return self.nlevel

    def set_nlevel(self, nlevel):
        self.nlevel = nlevel
        self.channels_channel_model_0.set_noise_voltage(self.nlevel)

    def get_report_callback(self):
        return self.report_callback

    def set_report_callback(self, report_callback):
        self.report_callback = report_callback

    def get_report_interval(self):
        return self.report_interval

    def set_report_interval(self, report_interval):
        self.report_interval = report_interval
        self.digital_probe_mpsk_snr_est_c_0.set_msg_nsample(int(self.report_interval * self.symb_rate))
        self.blocks_message_strobe_0.set_period(self.report_interval * 1000)

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.set_sps(self.samp_rate / self.symb_rate / self.decim)
        self.root_raised_cosine_filter_0.set_taps(firdes.root_raised_cosine(1, self.samp_rate /  self.decim, self.symb_rate, .5, self.rrc_order))
        self.osmosdr_source_0.set_sample_rate(self.samp_rate)
        self.blocks_multiply_const_vxx_0.set_k((self.samp_rate / self.decim / np.pi / 2 / 2, ))

    def get_symb_rate(self):
        return self.symb_rate

    def set_symb_rate(self, symb_rate):
        self.symb_rate = symb_rate
        self.set_sps(self.samp_rate / self.symb_rate / self.decim)
        self.root_raised_cosine_filter_0.set_taps(firdes.root_raised_cosine(1, self.samp_rate /  self.decim, self.symb_rate, .5, self.rrc_order))
        self.digital_probe_mpsk_snr_est_c_0.set_msg_nsample(int(self.report_interval * self.symb_rate))

    def get_sps(self):
        return self.sps

    def set_sps(self, sps):
        self.sps = sps
        self.digital_clock_recovery_mm_xx_0.set_omega(self.sps)

    def get_rrc_order(self):
        return self.rrc_order

    def set_rrc_order(self, rrc_order):
        self.rrc_order = rrc_order
        self.root_raised_cosine_filter_0.set_taps(firdes.root_raised_cosine(1, self.samp_rate /  self.decim, self.symb_rate, .5, self.rrc_order))

    def get_pfb_nfilt(self):
        return self.pfb_nfilt

    def set_pfb_nfilt(self, pfb_nfilt):
        self.pfb_nfilt = pfb_nfilt

    def get_channelizer_filter_taps(self):
        return self.channelizer_filter_taps

    def set_channelizer_filter_taps(self, channelizer_filter_taps):
        self.channelizer_filter_taps = channelizer_filter_taps
        self.set_channelizer_taps((1,) if self.decim == 1 else self.channelizer_filter_taps)

    def get_variable_cc_decoder_def_0(self):
        return self.variable_cc_decoder_def_0

    def set_variable_cc_decoder_def_0(self, variable_cc_decoder_def_0):
        self.variable_cc_decoder_def_0 = variable_cc_decoder_def_0

    def get_pll_loopbw(self):
        return self.pll_loopbw

    def set_pll_loopbw(self, pll_loopbw):
        self.pll_loopbw = pll_loopbw
        self.analog_pll_freqdet_cf_0.set_loop_bandwidth(2 * np.pi * self.pll_loopbw)
        self.analog_pll_freqdet_cf_0.set_max_freq(20 * np.pi * self.pll_loopbw)
        self.analog_pll_freqdet_cf_0.set_min_freq(-20 * np.pi * self.pll_loopbw)

    def get_pfb_taps(self):
        return self.pfb_taps

    def set_pfb_taps(self, pfb_taps):
        self.pfb_taps = pfb_taps

    def get_channelizer_taps(self):
        return self.channelizer_taps

    def set_channelizer_taps(self, channelizer_taps):
        self.channelizer_taps = channelizer_taps
        self.fir_channelizer.set_taps((self.channelizer_taps))


def argument_parser():
    parser = OptionParser(usage="%prog: [options]", option_class=eng_option)
    parser.add_option(
        "", "--decim", dest="decim", type="intx", default=1,
        help="Set Channelizer decimation factor [default=%default]")
    parser.add_option(
        "", "--device-args", dest="device_args", type="string", default='uhd,addr=192.168.10.3,otw_format=sc16',
        help="Set Device arguments [default=%default]")
    parser.add_option(
        "", "--digital-fc", dest="digital_fc", type="eng_float", default=eng_notation.num_to_str(0),
        help="Set Digital frequency offset (Hz) [default=%default]")
    parser.add_option(
        "", "--emwindir", dest="emwindir", type="string", default='.',
        help="Set EMWIN products directory [default=%default]")
    parser.add_option(
        "", "--carrier-fc", dest="carrier_fc", type="eng_float", default=eng_notation.num_to_str(137.5e3),
        help="Set Carrier frequency [default=%default]")
    parser.add_option(
        "", "--lritdir", dest="lritdir", type="string", default='.',
        help="Set LRIT products directory [default=%default]")
    parser.add_option(
        "", "--nlevel", dest="nlevel", type="eng_float", default=eng_notation.num_to_str(0),
        help="Set Digital noise level [0,1] [default=%default]")
    parser.add_option(
        "", "--report-callback", dest="report_callback", type="string", default='lambda x:x',
        help="Set Report callback [default=%default]")
    parser.add_option(
        "", "--report-interval", dest="report_interval", type="eng_float", default=eng_notation.num_to_str(10),
        help="Set Report interval (s) [default=%default]")
    parser.add_option(
        "", "--samp-rate", dest="samp_rate", type="eng_float", default=eng_notation.num_to_str(1e6),
        help="Set sample rate [default=%default]")
    parser.add_option(
        "", "--symb-rate", dest="symb_rate", type="eng_float", default=eng_notation.num_to_str(500e3),
        help="Set symbol rate [default=%default]")
    return parser


def main(top_block_cls=goes_lrit_emwin_nogui, options=None):
    if options is None:
        options, _ = argument_parser().parse_args()

    tb = top_block_cls(decim=options.decim, device_args=options.device_args, digital_fc=options.digital_fc, emwindir=options.emwindir, carrier_fc=options.carrier_fc, lritdir=options.lritdir, nlevel=options.nlevel, report_callback=options.report_callback, report_interval=options.report_interval, samp_rate=options.samp_rate, symb_rate=options.symb_rate)
    tb.start()
    tb.wait()


if __name__ == '__main__':
    main()
