#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 The Aerospace Corporation.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from time import sleep
from math import copysign

KHZ, MHZ, GHZ = 1e3, 1e6, 1e9

def afoc(fcn_meas_foff, fcn_set_foff, running=1, logger=None, 
         interval=1, off_tol=500, on_tol=5000):
    """Performs automatic frequency offset correction.
    
    The afoc function uses two callables to measure and set the frequency offset.
    Optionally operate in a separate thread to continually monitor and correct 
    frequency offset.
    
    Required parameters:
        fcn_set_foff    callable    No arguments, returns an offset measurement
        fcn_set_foff    callable    Freq argument, sets the offset estimate
        
    Optional parameters:
    running     {threading.Event | positive integer}    Controls loop operation (Default: 1)
        When a threading.Event, the update loop will run as soon as running is
        set until it is unset.  If an integer, running specifies the number of
        update rounds until exit.
    logger      logging.logger      Optional logger for log output (Default: None)
    interval    positive float      Seconds between measurements  (Default: 1)
    off_tol     positive number     Tolerance to turn updates off (Default: 500)
        Updates continue as long as the absolute value of the measurement 
        exceeds this value.
    on_tol      positive number     Tolerance to turn on updates (Default: 5000)
        Updates will resume if the absolute value of the measurement exceeds
        this value.  Along with off_tol, this parameter can be used to define
        a hysteresis to reduce the number of frequency updates. (Default: 
    """
    finite_loop = False                 # run for fixed number of iterations?
    isoff = False                       # updates are off?
    tol = off_tol                       # current offset tolerance
    try:
        running.wait()                  # wait until running is set
    except AttributeError:              # running not a threading.event, should be int > 0
        finite_loop = True
    while (finite_loop and running > 0) or running.is_set():
        sleep(interval)
        foff = fcn_meas_foff()          # measurement returns foffset (Hz)
        logger and logger.debug("Measured carrier offset = %.3f KHz", foff / KHZ)
        if abs(foff) > tol:
            tol = off_tol               # offset to turn updates off
            isoff = False               # updates are on
        else:
            tol = on_tol                # offset to turn updates on
            isoff = True                # updates are off
        not isoff and fcn_set_foff(foff)    # update freq offset
        if finite_loop: running -= 1    # one iteration down

# guard band > norm'd signal bandwidth
def make_foffset_splitter(fs, guard_band=0.1):
    """Returns a function that decomposes given foffset into deltas for HW and
    digital LO"""
    
    def foffset_split(old, fo, clip=False):
        band_edge = 0.5 - guard_band
        if clip:
           fo = copysign(1, old + fo) * (min(abs(fo + old), fs * band_edge) - old)
        over_band_edge = abs(old + fo) / fs > band_edge
        if over_band_edge: 
            return old + fo, 0         # HW LO delta is total offset
        return 0, fo                   # return HW and digital LO deltas
    return foffset_split


